package com.sun.xml.internal.ws.api.streaming;

import com.sun.xml.internal.ws.encoding.HasEncoding;
import com.sun.xml.internal.ws.util.xml.XMLStreamWriterFilter;
import javax.xml.ws.WebServiceException;
import java.lang.reflect.InvocationTargetException;
import com.sun.istack.internal.Nullable;
import javax.xml.transform.stream.StreamResult;
import java.lang.reflect.Method;
import com.sun.xml.internal.ws.streaming.XMLReaderException;
import javax.xml.stream.XMLStreamException;
import java.util.logging.Level;
import java.io.StringWriter;
import javax.xml.stream.XMLOutputFactory;
import com.sun.istack.internal.NotNull;
import javax.xml.stream.XMLStreamWriter;
import java.io.OutputStream;
import java.util.logging.Logger;

public abstract class XMLStreamWriterFactory
{
    private static final Logger LOGGER;
    private static volatile ContextClassloaderLocal<XMLStreamWriterFactory> writerFactory;
    
    public XMLStreamWriterFactory() {
    }
    
    public abstract XMLStreamWriter doCreate(OutputStream p0);
    
    public abstract XMLStreamWriter doCreate(OutputStream p0, String p1);
    
    public abstract void doRecycle(XMLStreamWriter p0);
    
    public static void recycle(XMLStreamWriter r) {
        get().doRecycle(r);
    }
    
    @NotNull
    public static XMLStreamWriterFactory get() {
        return (XMLStreamWriterFactory)XMLStreamWriterFactory.writerFactory.get();
    }
    
    public static void set(@NotNull XMLStreamWriterFactory f) {
        if (f == null) {
            throw new IllegalArgumentException();
        }
        XMLStreamWriterFactory.writerFactory.set(f);
        return;
    }
    
    public static XMLStreamWriter create(OutputStream out) {
        return get().doCreate(out);
    }
    
    public static XMLStreamWriter create(OutputStream out, String encoding) {
        return get().doCreate(out, encoding);
    }
    
    @Deprecated
    public static XMLStreamWriter createXMLStreamWriter(OutputStream out) {
        return create(out);
    }
    
    @Deprecated
    public static XMLStreamWriter createXMLStreamWriter(OutputStream out, String encoding) {
        return create(out, encoding);
    }
    
    @Deprecated
    public static XMLStreamWriter createXMLStreamWriter(OutputStream out, String encoding, boolean declare) {
        return create(out, encoding);
    }
    
    static {
        LOGGER = Logger.getLogger(XMLStreamWriterFactory.class.getName());
        XMLStreamWriterFactory.writerFactory = new ContextClassloaderLocal<XMLStreamWriterFactory>() {
            @Override
            protected XMLStreamWriterFactory initialValue() {
                XMLOutputFactory xof = null;
                if (Boolean.getBoolean(XMLStreamWriterFactory.class.getName() + ".woodstox")) {
                    try {
                        xof = (XMLOutputFactory)Class.forName("com.ctc.wstx.stax.WstxOutputFactory").newInstance();
                    }
                    catch (Exception ex3) {}
                }
                if (xof == null) {
                    xof = XMLOutputFactory.newInstance();
                }
                XMLStreamWriterFactory f = null;
                if (!Boolean.getBoolean(XMLStreamWriterFactory.class.getName() + ".noPool")) {
                    try {
                        Class<?> clazz = xof.createXMLStreamWriter(new StringWriter()).getClass();
                        if (clazz.getName().startsWith("com.sun.xml.internal.stream.")) {
                            f = new XMLStreamWriterFactory.Zephyr(xof, clazz);
                        }
                    }
                    catch (XMLStreamException ex) {
                        Logger.getLogger(XMLStreamWriterFactory.class.getName()).log(Level.INFO, null, ex);
                    }
                    catch (NoSuchMethodException ex2) {
                        Logger.getLogger(XMLStreamWriterFactory.class.getName()).log(Level.INFO, null, ex2);
                    }
                }
                if (f == null && xof.getClass().getName().equals("com.ctc.wstx.stax.WstxOutputFactory")) {
                    f = new NoLock(xof);
                }
                if (f == null) {
                    f = new Default(xof);
                }
                if (XMLStreamWriterFactory.LOGGER.isLoggable(Level.FINE)) {
                    XMLStreamWriterFactory.LOGGER.log(Level.FINE, "XMLStreamWriterFactory instance is = {0}", f);
                    return f;
                }
                return f;
            }
        };
    }
    
    public static final class Default extends XMLStreamWriterFactory
    {
        private final XMLOutputFactory xof;
        
        public Default(XMLOutputFactory xof) {
            this.xof = xof;
        }
        
        @Override
        public XMLStreamWriter doCreate(OutputStream out) {
            return this.doCreate(out, "UTF-8");
        }
        
        @Override
        public synchronized XMLStreamWriter doCreate(OutputStream out, String encoding) {
            try {
                XMLStreamWriter writer = this.xof.createXMLStreamWriter(out, encoding);
                return new XMLStreamWriterFactory.HasEncodingWriter(writer, encoding);
            }
            catch (XMLStreamException e) {
                throw new XMLReaderException("stax.cantCreate", new Object[] { e });
            }
        }
        
        @Override
        public void doRecycle(XMLStreamWriter r) {
        }
    }
    
    public static final class Zephyr extends XMLStreamWriterFactory
    {
        private final XMLOutputFactory xof;
        private final ThreadLocal<XMLStreamWriter> pool;
        private final Method resetMethod;
        private final Method setOutputMethod;
        private final Class zephyrClass;
        
        public static XMLStreamWriterFactory newInstance(XMLOutputFactory xof) {
            try {
                Class<?> clazz = xof.createXMLStreamWriter(new StringWriter()).getClass();
                if (!clazz.getName().startsWith("com.sun.xml.internal.stream.")) {
                    return null;
                }
                return new XMLStreamWriterFactory.Zephyr(xof, clazz);
            }
            catch (XMLStreamException e) {
                return null;
            }
            catch (NoSuchMethodException e2) {
                return null;
            }
        }
        
        private Zephyr(XMLOutputFactory xof, Class clazz) throws NoSuchMethodException {
            this.pool = new ThreadLocal();
            this.xof = xof;
            this.zephyrClass = clazz;
            this.setOutputMethod = clazz.getMethod("setOutput", new Class[] { StreamResult.class, String.class });
            this.resetMethod = clazz.getMethod("reset", new Class[0]);
        }
        
        @Nullable
        private XMLStreamWriter fetch() {
            XMLStreamWriter sr = (XMLStreamWriter)this.pool.get();
            if (sr == null) {
                return null;
            }
            this.pool.set(null);
            return sr;
        }
        
        @Override
        public XMLStreamWriter doCreate(OutputStream out) {
            return this.doCreate(out, "UTF-8");
        }
        
        @Override
        public XMLStreamWriter doCreate(OutputStream out, String encoding) {
            XMLStreamWriter xsw = this.fetch();
            if (xsw != null) {
                try {
                    this.resetMethod.invoke(xsw, new Object[0]);
                    this.setOutputMethod.invoke(xsw, new Object[] { new StreamResult(out), encoding });
                }
                catch (IllegalAccessException e2) {
                    throw new XMLReaderException("stax.cantCreate", new Object[] { e2 });
                }
                catch (InvocationTargetException e3) {
                    throw new XMLReaderException("stax.cantCreate", new Object[] { e3 });
                }
            }
            else {
                try {
                    xsw = this.xof.createXMLStreamWriter(out, encoding);
                }
                catch (XMLStreamException e) {
                    throw new XMLReaderException("stax.cantCreate", new Object[] { e });
                }
            }
            return new XMLStreamWriterFactory.HasEncodingWriter(xsw, encoding);
        }
        
        @Override
        public void doRecycle(XMLStreamWriter r) {
            if ((r instanceof XMLStreamWriterFactory.HasEncodingWriter)) {
                r = ((XMLStreamWriterFactory.HasEncodingWriter)r).getWriter();
            }
            if (this.zephyrClass.isInstance(r)) {
                try {
                    r.close();
                }
                catch (XMLStreamException e) {
                    throw new WebServiceException(e);
                }
                this.pool.set(r);
            }
            if ((r instanceof XMLStreamWriterFactory.RecycleAware)) {
                ((XMLStreamWriterFactory.RecycleAware)r).onRecycled();
            }
        }
    }
    
    public static final class NoLock extends XMLStreamWriterFactory
    {
        private final XMLOutputFactory xof;
        
        public NoLock(XMLOutputFactory xof) {
            this.xof = xof;
        }
        
        @Override
        public XMLStreamWriter doCreate(OutputStream out) {
            return this.doCreate(out, "utf-8");
        }
        
        @Override
        public XMLStreamWriter doCreate(OutputStream out, String encoding) {
            try {
                XMLStreamWriter writer = this.xof.createXMLStreamWriter(out, encoding);
                return new XMLStreamWriterFactory.HasEncodingWriter(writer, encoding);
            }
            catch (XMLStreamException e) {
                throw new XMLReaderException("stax.cantCreate", new Object[] { e });
            }
        }
        
        @Override
        public void doRecycle(XMLStreamWriter r) {
        }
    }
    
    public static class HasEncodingWriter extends XMLStreamWriterFilter implements HasEncoding
    {
        private final String encoding;
        
        HasEncodingWriter(XMLStreamWriter writer, String encoding) {
            super(writer);
            this.encoding = encoding;
        }
        
        public String getEncoding() {
            return this.encoding;
        }
        
        public XMLStreamWriter getWriter() {
            return this.writer;
        }
    }
    
    public interface RecycleAware
    {
        public abstract void onRecycled();
    }
}
